Type: Package
Package: dbplyr
Title: A 'dplyr' Back End for Databases
Version: 2.6.0
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre")),
    person("Maximilian", "Girlich", role = "aut"),
    person("Edgar", "Ruiz", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: A 'dplyr' back end for databases that allows you to work with
    remote database tables as if they are in-memory data frames.  Basic
    features work with any database that has a 'DBI' back end; more
    advanced features require 'SQL' translation to be provided by the
    package author.
License: MIT + file LICENSE
URL: https://dbplyr.tidyverse.org/, https://github.com/tidyverse/dbplyr
BugReports: https://github.com/tidyverse/dbplyr/issues
Depends: R (>= 4.1)
Imports: blob (>= 1.2.0), cli (>= 3.6.1), DBI (>= 1.1.3), dplyr (>=
        1.1.2), glue (>= 1.6.2), lifecycle (>= 1.0.3), magrittr,
        methods, pillar (>= 1.9.0), purrr (>= 1.0.1), R6 (>= 2.2.2),
        rlang (>= 1.1.1), tibble (>= 3.2.1), tidyr (>= 1.3.0),
        tidyselect (>= 1.2.1), utils, vctrs (>= 0.6.3), withr (>=
        2.5.0)
Suggests: adbcdrivermanager, adbcsqlite, adbi, bit64, covr, knitr,
        Lahman, nycflights13, odbc (>= 1.4.2), RJDBC, RMariaDB (>=
        1.2.2), rmarkdown, RPostgres (>= 1.4.5), RPostgreSQL, RSQLite
        (>= 2.3.8), testthat (>= 3.1.10)
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: TRUE
Encoding: UTF-8
Language: en-gb
Collate: 'verb-copy-inline.R' 'verb-copy-to.R' 'db-sql.R' 'db.R'
        'utils-check.R' 'import-standalone-types-check.R'
        'import-standalone-obj-type.R' 'utils.R' 'sql.R' 'escape.R'
        'translate-sql-cut.R' 'translate-sql-string.R'
        'translate-sql-aggregate.R' 'translate-sql-scalar.R'
        'translate-sql-helpers.R' 'translate-sql-window.R'
        'translate-sql-conditional.R' 'backend-.R' 'backend-access.R'
        'backend-adbc.R' 'backend-db2.R' 'backend-hana.R'
        'backend-hive.R' 'backend-impala.R' 'backend-jdbc.R'
        'backend-mssql.R' 'backend-mysql.R' 'backend-odbc.R'
        'backend-oracle.R' 'backend-postgres.R'
        'backend-postgres-old.R' 'backend-redshift.R'
        'backend-snowflake.R' 'backend-spark-sql.R' 'backend-sqlite.R'
        'backend-teradata.R' 'backward-compatibility.R'
        'bind-queries.R' 'data-cache.R' 'data-lahman.R'
        'data-nycflights13.R' 'db-io.R' 'dbplyr.R' 'ident.R'
        'import-standalone-s3-register.R' 'join-by-compat.R'
        'join-cols-compat.R' 'lazy-ops.R' 'memdb.R' 'optimise-utils.R'
        'progress.R' 'query-base.R' 'query-join.R' 'query-rf-join.R'
        'query-select.R' 'query-semi-join.R' 'query-set-op.R'
        'query-union.R' 'query.R' 'remote.R' 'rows.R' 'schema.R'
        'sql-build.R' 'sql-clause.R' 'sql-dialect.R' 'sql-glue.R'
        'sql-quote.R' 'sql-superseded.R' 'src-sql.R' 'src_dbi.R'
        'table-name.R' 'tbl-lazy.R' 'tbl-sql.R' 'tidyeval-across.R'
        'tidyeval.R' 'translate-sql.R' 'utils-format.R'
        'verb-arrange.R' 'verb-collapse.R' 'verb-collect.R'
        'verb-compute.R' 'verb-count.R' 'verb-distinct.R'
        'verb-do-query.R' 'verb-do.R' 'verb-expand.R' 'verb-explain.R'
        'verb-fill.R' 'verb-filter.R' 'verb-group_by.R' 'verb-head.R'
        'verb-joins.R' 'verb-mutate.R' 'verb-pivot-longer.R'
        'verb-pivot-wider.R' 'verb-pull.R' 'verb-select.R'
        'verb-set-ops.R' 'verb-slice.R' 'verb-summarise.R'
        'verb-uncount.R' 'verb-window.R' 'with-dialect.R' 'zzz.R'
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-17 12:33:03 UTC; hadleywickham
Author: Hadley Wickham [aut, cre],
  Maximilian Girlich [aut],
  Edgar Ruiz [aut],
  Posit Software, PBC [cph, fnd]
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2026-06-17 21:50:07 UTC
Built: R 4.5.2; ; 2026-06-17 23:13:16 UTC; unix
