test_that("queries generated by select() don't alias unnecessarily", {
  lf_build <- lazy_frame(x = 1, y = 1) |>
    select(x) |>
    sql_build()

  lf_render <- sql_render(lf_build, con = dialect_ansi())
  expect_snapshot(lf_render)
})

test_that("queries generated by select() don't alias unnecessarily", {
  expect_snapshot(error = TRUE, lazy_frame(x = 1, y = 1) |> select())
})
