test_that("zero length inputs return correct clases", {
  expect_s3_class(ident(), "ident")
})

test_that("ident quotes", {
  con <- dialect_ansi()
  x1 <- ident("x")

  expect_equal(escape(x1, con = con), sql('"x"'))
})

test_that("can format ident", {
  expect_snapshot(ident())
})

test_that("as_ident_or_sql() accepts character subclasses like glue", {
  expect_equal(as_ident_or_sql(glue::glue("x")), ident("x"))
})

test_that("as_ident_or_sql() gives informative error for non-character", {
  expect_snapshot(as_ident_or_sql(1), error = TRUE)
})
