# PostgreSQL

## Install

First install PostgreSQL, create a data directory, and create a default database.

```
brew install postgresql
export PGDATA=~/db/postgres # set this globally somewhere
initdb
createdb test

psql -c "CREATE USER postgres WITH PASSWORD 'password';"
psql -c "ALTER USER postgres WITH SUPERUSER;"
```

## Start

```
pg_ctl start
```

## Connect

```{r, eval = FALSE}
library(DBI)
con <- dbConnect(RPostgres::Postgres(), dbname = "hadley")
dbListTables(con)

con <- dbConnect(
  RPostgres::Postgres(), 
  dbname = "test", 
  user = "postgres", 
  password = "passowrd"
)
```
