# can print lazy_select_query

    Code
      lazy_select_query(x = lf$lazy_query, select = quos(x_mean = mean(x), y2 = y),
      where = quos(y > 1, x == y - 2), group_by = quos("x"))
    Output
      <SQL SELECT>
      From:
        <table_path> `df`
      Select:   x_mean = mean(x), y2 = y
      Where:    y > 1, x == y - 2
      Group by: "x"

