# useful error for 1st edition

    Code
      check_2ed(con)
    Condition
      Error:
      ! <Test> uses dbplyr's 1st edition interface, which is no longer supported.
      i Please contact the maintainer of the package for a solution.

