# queries generated by select() don't alias unnecessarily

    Code
      lf_render
    Output
      <SQL> SELECT "x"
      FROM "df"

---

    Code
      select(lazy_frame(x = 1, y = 1))
    Condition
      Error in `sql_clause_select()`:
      ! Query contains no columns

