% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/backward-compatibility.R
\name{sql_optimise}
\alias{sql_optimise}
\title{Optimise a SQL query}
\usage{
sql_optimise(x, con = NULL, ..., subquery = FALSE)
}
\description{
No longer used.
}
\keyword{internal}
