% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/backend-hive.R, R/backend-impala.R,
%   R/backend-mssql.R, R/simulate.R
\name{simulate_hive}
\alias{simulate_hive}
\alias{simulate_impala}
\alias{simulate_mssql}
\alias{simulate_dbi}
\title{Simulate database connections}
\usage{
simulate_hive()

simulate_impala()

simulate_mssql(version = "15.0")

simulate_dbi(class = character(), ...)
}
\description{
These functions generate S3 objects that have been designed to simulate
the action of a database connection, without actually having the database
available. Obviously, this simulation can only be incomplete, but most
importantly it allows us to simulate SQL generation for any database without
actually connecting to it.
}
\details{
Simulated SQL always quotes identifies with \code{`x`}, and strings with
\code{'x'}.
}
\keyword{internal}
